<?php

namespace JimsCustomerBooking\RestfulAPI;

use Exception;
use JimsCustomerBooking\Admin\BackendPages\EmailSubscriptionSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Integrations\JimsResourcesIntegration;
use WP_REST_Request;
use WP_REST_Response;

class EmailSubscriptionController extends RestController
{
    protected $route = 'email-subscription';

    public function register_routes(){
        register_rest_route($this->get_namespace(), '/' . $this->get_route () . '/create', [
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => array($this, 'subscribe'),
                'args' => [
                    'email' => [
                        'required' => true,
                        'validate_callback' => function($param, $request, $key){
                            return is_email($param);
                        }
                    ]
                ],
                'permission_callback' => '__return_true',
            ]
        ]);

        register_rest_route($this->get_namespace(), '/' . $this->get_route () . '/subscribed', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => array($this, 'isSubscribed'),
                'args' => [
                    'email' => [
                        'required' => true,
                        'validate_callback' => function($param, $request, $key){
                            return is_email($param);
                        }
                    ]
                ],
                'permission_callback' => '__return_true',
            ]
        ]);

        register_rest_route($this->get_namespace(), '/' . $this->get_route () . '/get', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => array($this, 'get_subscription'),
                'permission_callback' => '__return_true',
            ]
        ]);
    }


    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response
     */
    public function get_subscription(WP_REST_Request $request): WP_REST_Response
    {
        $options = OptionHelper::get(EmailSubscriptionSettingsPage::$option_name, []);

        return $this->response([
            'email' => $options['email'] ?? '',
            'name' => $options['name'] ?? ''
        ]);
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response
     * @throws Exception
     */
    public function subscribe(WP_REST_Request $request): WP_REST_Response {
        $email = $request->get_param('email');
        $name = $request->get_param('name');

        $saved = OptionHelper::update(EmailSubscriptionSettingsPage::$option_name, [
            'email' => $email,
            'name' => $name
        ]);

        $integration = new JimsResourcesIntegration();
        $response = $integration->subscribe($email, $request->get_param('name') );

        return !empty($response) ? $this->response([
            'status' => true,
            'saved' => $saved,
            'message' => 'Email subscribed successfully'
        ]) : $this->response([
            'status' => false,
            'saved' => $saved,
            'message' => 'Failed to subscribe email'
        ], 400);
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response
     */
    public function isSubscribed(WP_REST_Request $request): WP_REST_Response {
        $integration = new JimsResourcesIntegration();

        return $this->response([
            'status' => $integration->isSubscribed($request->get_param('email'))
        ]);
    }
}