<?php

namespace JimsGroup\Utilities;

/**
 * Class Logger
 * @package JimsGroup\Utilities
 */
class Logger
{
    /**
     * @var string
     */
    private static $mode = 'default'; // 'default' or 'file'


    /**
     * @var string
     */
    private static $custom_file = 'log.txt';


    /**
     * @var string
     */
    private static $root_dir = __DIR__;


    /**
     * @var float
     */
    private static $time = 0;

    /**
     * @param $message
     * @return bool
     */
    public static function log($message, $type = 'generic'):bool
    {
        if(self::$root_dir === __DIR__ && defined('ABSPATH')) {
            self::$root_dir = constant('ABSPATH');
        }

        $log = self::getLogFromMessage($message, $type);

        if (self::$mode === 'default') {
            return error_log($log);
        } else {
            return self::custom_log($log);
        }
    }

    /**
     * @param $message
     * @return bool
     */
    public static function info($message): bool
    {
        return self::log($message, 'info');
    }

    /**
     * @param $message
     * @return bool
     */
    public static function error($message): bool
    {
        return self::log($message, 'error');
    }


    /**
     * @param $message
     * @return bool
     */
    public static function custom_log($message): bool
    {
        return (bool) file_put_contents( implode('/', [self::$root_dir, self::$custom_file]), $log, FILE_APPEND);
    }

    /**
     * @param $message
     * @return string
     */
    public static function getLogFromMessage($message, $type = 'info'): string
    {
        $value_delimiter = ' - ';

        if( is_array($message) || is_object($message) ) {
            $message = json_encode($message);
        }

        if(!is_string($message) ){
            $message = var_export($message, true);
        }

        return date('Y-m-d H:i:s') . $value_delimiter . $type . $value_delimiter . $message . PHP_EOL;
    }

    /**
     * @param string $stage
     * @return float
     */
    public static function timer($stage = 'start'): float
    {
        if ($stage === 'start') {
            return self::$time = microtime(true);
        } else {
            return self::$time > 0 ? microtime(true) - self::$time : 0;
        }
    }

    /**
     * @param $mode
     * @return void
     */
    public static function setMode($mode):void
    {
        self::$mode = $mode;
    }

    /**
     * @param $file
     * @return void
     */
    public static function setCustomFile($file):void
    {
        self::$custom_file = $file;
    }

    /**
     * @param string $root_dir
     * @return void
     */
    public static function setRootDir(string $root_dir): void
    {
        self::$root_dir = $root_dir;
    }
}